#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
  @Author: zhangzongliang
  @Date: 2022-08-09 15:57:50
  @Description:
  @Email: noaghzil@gmail.com
  @Last Modified time: 2022-08-16 17:44:00
"""
from min_tbox import filter_dict

def test_filter_dict():
    user_info = {"title": 8, "hospital": "测试医院", "department": "内科", "gender": 1}
    u_format = ["title", "name", "hospital"]
    a = filter_dict(user_info, u_format)
    assert set(a.keys()).difference(u_format) == set()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
  @Author: zhangzongliang
  @Date: 2022-08-09 15:33:43
  @Description:
  @Email: noaghzil@gmail.com
  @Last Modified time: 2022-08-16 17:43:49
"""
import time
from datetime import datetime
from turtle import pd
import redis
import pytest
from min_tbox import ex_cache

@pytest.fixture(scope="session")
def rds_client():
    rds_params = {
        "host": '127.0.0.1',
        "port": 6379,
        "db": 0,
        "password": None,
    }
    rds = redis.StrictRedis(
        **rds_params,
        socket_timeout=5,
        socket_connect_timeout=5,
        decode_responses=True,
    )
    return rds

def test_cache(rds_client):
    
    class csr:

        num = 0

        @classmethod
        @ex_cache(rds_client, delay=5)
        def add_sum(self, a, b, c=1):
            self.num += 1
            print("calculate time:", datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
            return a + b + c

    for _ in range(30):
        time.sleep(1)
        print(csr.add_sum(1, 2))
    assert csr.num <= 6 and  csr.num > 0



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
  @Author: zhangzongliang
  @Date: 2022-08-09 15:39:15
  @Description:
  @Email: noaghzil@gmail.com
  @Last Modified time: 2022-08-09 15:49:25
"""



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
  @Author: zhangzongliang
  @Date: 2022-08-09 15:03:07
  @Description: 缓存装饰器
  @Email: noaghzil@gmail.com
  @Last Modified time: 2022-08-09 16:01:49
"""
import json
import logging
import inspect
import pickle
import wrapt
from .tools import ObjectEncoder

logger = logging.getLogger(__name__)


class BaseCache(object):
    def __init__(self, cache, delay=60):
        self.cache = cache
        self.delay_s = delay

    def _dumps(self, info):
        if not isinstance(info, str):
            info = json.dumps(info, cls=ObjectEncoder)
        return info

    def _loads(self, info):
        try:
            if isinstance(info, str):
                info = json.loads(info)
        except Exception:
            pass
        return info

    def _one_key(self):
        sig = inspect.signature(self.func).parameters
        f_name = '.'.join([self.func.__module__, self.func.__name__])
        pms_args = zip(list(sig.keys()), self.args)
        pms_d = dict(pms_args)
        pms_d.update(self.kwargs)
        for key in (sig.keys() - pms_d.keys()):
            pms_d[key] = sig[key].default
        no = pickle.dumps((f_name, pms_d))
        logger.info(f"cached|_one_key: {f_name}{pms_d}")
        return no


class ex_cache(BaseCache):
    @wrapt.decorator
    def __call__(self, func, instance, args, kwargs):
        self.func = func
        self.args, self.kwargs = args, kwargs
        res = self._loads(self.cache.get(self._one_key()))
        if not res:
            logger.info("cached: no thing")
            res = self.do_func()
        logger.info("cached: call thing")
        return res

    def do_func(self):
        res = self.func(*self.args, **self.kwargs)
        self.cache.set(self._one_key(), self._dumps(res), ex=self.delay_s)
        return res


class asyc_cache(BaseCache):
    @wrapt.decorator
    async def __call__(self, func, instance, args, kwargs):
        self.func = func
        self.args, self.kwargs = args, kwargs
        res = self._loads(self.cache.get(self._one_key()))
        if not res:
            logger.info("cached: no thing")
            res = await self.do_func()
        logger.info("cached: call thing")
        return res

    async def do_func(self):
        res = await self.func(*self.args, **self.kwargs)
        self.cache.set(self._one_key(), self._dumps(res), ex=self.delay_s)
        return res



# -*- coding: utf-8 -*-
# @Author: Noaghzil
# @Date:   2022-08-15 10:38:25
# @Last Modified by:   Noaghzil
# @Description: 封装http请求
# @Last Modified time: 2022-08-16 17:18:14
from email import header
import json
import logging
import time

import aiohttp
from aiohttp import ClientResponse

logger = logging.getLogger(__name__)


def _try_to_json(data) -> str:
    try:
        return json.dumps(data)[:2000]
    except:
        return str(data)[:2000]


class Http:

    def __init__(self, timeout=10):
        self.timeout = aiohttp.ClientTimeout(total=timeout)

    async def request(self, url, method, headers=None, data=None, content_type='json'):
        """
            网络请求
            :param url: 地址
            :param method: http方法
            :param data: 参数
            :param content_type: 返回类型
            :return: 结果
        """
        data = data or {}
        headers = headers or {"Content-Type": "application/json"}
        result = {}
        start = time.time()
        now_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        logger.info("Http.%s time: [%s], request to [%s] with data [%s]" % (method, url, _try_to_json(data), now_time))
        pms = {"method": method, "url": url, "headers": headers}
        async with aiohttp.ClientSession(timeout=self.timeout) as session:
            if 'application/json' in headers.get('Content-Type'):
                pms["json"] = data
            else:
                pms["params"] = data
            async with session.request(**pms) as response:
                result = await self.get_result(response, content_type)
                spend = time.time() - start
                if spend > 3:
                    logger.warning("Http.%s time: [%s], response is success but slow, spend: [%s]" %
                                   (method, now_time, spend))
                else:
                    logger.info("Http.%s time: [%s]" % (method, spend))
        return result

    async def get(self, url, data=None, headers=None, content_type='json') -> dict:
        """
            Http get方法
            :param url: 地址
            :param data: 参数
            :param content_type: 返回类型
            :return: 结果
        """
        result = await self.request(url, 'GET', headers, data, content_type)
        return result

    async def post(self, url, data=None, headers=None, content_type='json') -> dict:
        """
            Http post方法
            :param url: 地址
            :param data: 参数
            :param content_type: 返回类型
            :return: 结果
        """
        result = await self.request(url, 'POST', headers, data, content_type)
        return result

    async def get_result(self, response: ClientResponse, content_type='json'):
        """
            获取返回结果
            :param response: 响应
            :param content_type: 返回类型
            :return: 结果
        """
        result = {}
        try:
            if content_type == 'json':
                result = await response.json(content_type=None)
            else:
                result = await response.read()
            logger.info("Http.get_result is [%s]", _try_to_json(result))
        except json.decoder.JSONDecodeError as e:
            r = await response.read()
            logger.exception("Error while Http.get_result is [%s], return [%s]", e, _try_to_json(r))
            raise ValueError("Error while Http.get_result is [%s], return [%s]", e, _try_to_json(r))
        return result



# -*- coding: utf-8 -*-
# @Author: Noaghzil
# @Date:   2022-08-15 11:12:05
# @Last Modified by:   Noaghzil
# @Last Modified time: 2022-08-16 17:43:57
import pytest
from min_tbox.api import Http


@pytest.fixture(scope="session")
def client():
    return Http()


@pytest.mark.asyncio
async def test_http_get(client: Http):
    """
     Http get方法,  测试用例
    """
    url_path = "https://xxx.com/soi/polling?chat_id=17654642284957696&uid=200&user_type=1&action=get_msgs"
    result = await client.get(url_path)
    assert result['code'] == "200"
    assert result['status'] == 0
    assert len(result['update_details']) == 1
    assert 1 == 1


@pytest.mark.asyncio
async def test_http_post(client: Http):
    """
     Http post方法, 测试用例
    """
    url_path = "https://xxx.com/chat_api/member/chat_label"
    params = {
        "chat_id": 96,
        "app_id": "5fa769c142e6df8c6a36b3d3",
        "uid": "123",
        "user_type": 1,
        "chat_label": "docker交流群"
    }
    result = await client.post(url_path, data=params)
    assert result['status'] == 0



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
  @Author: zhangzongliang
  @Date: 2022-08-31 16:54:08
  @Description: 命名转换
  @Email: noaghzil@gmail.com
  @Last Modified time: 2022-08-31 16:55:24
"""
import re


def to_snake_case(x):
    """转下划线命名"""
    return re.sub('(?<=[a-z])[A-Z]|(?<!^)[A-Z](?=[a-z])', '_\g<0>', x).lower()


def to_camel_case(x):
    """转驼峰法命名"""
    return re.sub('_([a-zA-Z])', lambda m: (m.group(1).upper()), x)


def to_upper_camel_case(x):
    """转大驼峰法命名"""
    s = re.sub('_([a-zA-Z])', lambda m: (m.group(1).upper()), x)
    return s[0].upper() + s[1:]


def to_lower_camel_case(x):
    """转小驼峰法命名"""
    s = re.sub('_([a-zA-Z])', lambda m: (m.group(1).upper()), x)
    return s[0].lower() + s[1:]



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
  @Author: zhangzongliang
  @Date: 2022-08-09 15:07:47
  @Description: 基础工具包
  @Email: noaghzil@gmail.com
  @Last Modified time: 2022-08-15 17:19:38
"""

import json
from datetime import date
from datetime import datetime
from decimal import Decimal


class ObjectEncoder(json.JSONEncoder):
    """
      日期解密工具
    """
    def default(self, obj):
        if isinstance(obj, bytes):
            return obj.decode()
        if isinstance(obj, datetime):
            return obj.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(obj, date):
            return obj.strftime("%Y-%m-%d")
        if isinstance(obj, Decimal):
            return int(obj)
        return json.JSONEncoder.default(self, obj)


def filter_dict(params, include_keys):
    """
      按keys，过滤字典中的字段
    """
    return isinstance(params, dict) and isinstance(include_keys, list) and dict(
        filter(lambda x: x[0] in include_keys, params.items())) or {}



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
  @Author: zhangzongliang
  @Date: 2022-08-09 11:37:54
  @Description:
  @Email: noaghzil@gmail.com
  @Last Modified time: 2022-08-16 17:45:08
"""

from setuptools import find_packages, setup

setup(
    name="min_tbox",
    version="0.0.3",
    description='some kit tool ',
    author='noaghzil',
    author_email='noaghzil@gmail.com',
    packages=find_packages(),
    install_requires=["redis", "pytest", "pytest-asyncio", "aiohttp", "wrapt"],
)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
  @Author: zhangzongliang
  @Date: 2022-08-09 15:36:18
  @Description:
  @Email: noaghzil@gmail.com
  @Last Modified time: 2022-08-09 16:03:51
"""


from .cache_tools import ex_cache, asyc_cache
from .tools import ObjectEncoder, filter_dict


__all__ = ["cache_tools", "asyc_cache", "filter_dict", "ObjectEncoder"]
